/*
    file: mGNFAtoDFA_mG.cpp
    Project: miniGrail
    License: LGPLv2.1 http://www.gnu.org/licenses/lgpl-2.1.html

    Created by: by Rui Zhou
    Maintainer: Cezar Campeanu <ccampeanu@upei.ca>

    Modified by Brandon Stewart July 2015
	  - Added namespace minigrail

 * Re- implement the nfatodfa set algorithms with the "MachineCat algorithm"
 *
 *
 *
 *
 This use the very basic set algorithm to transform an NFA into a DFA
 */

namespace minigrail
{

void mGNFAtoDFA_mG(mGNFA& nfa, mGDFA& dfa) //question  why can not const?
{

//clear the DFA to output
	dfa.clear();
//attach
	nfa.attach();

//gather the labels in the NFA
	mGSet<int> alphabets;
	nfa.gatherLabels(alphabets);

// the sets that will be the DFA states
	mGArray<mGSet<mGState> > setsToProcess;
	mGArray<mGInt> bigInts;

	int index = 0; // current index in the sets to process

	setsToProcess += nfa.getStartStates();
	bigInts += mGInt(nfa.getStartStates());

//calculate
	while (index < setsToProcess.size())
	{

///get all involved transitions
		mGSet<mGTransition> transitionsInvolved;

		for (int a = 0; a < setsToProcess[index].size(); a++) //look at  each set of states
		{
			transitionsInvolved.disjoint_union(
					nfa.getTransitionsStartFrom((setsToProcess[index])[a]));
		}

//store them by labels
		mGMap<int, mGSet<mGState> > labelSets;

		for (int a = 0; a < transitionsInvolved.size(); a++)
		{
			labelSets[transitionsInvolved[a].getLabel()] +=
					transitionsInvolved[a].getEnd();
		}

//now , look at the dictionary (map), add new Sets to setsToProcess as well as
//add transitions to DFA
		for (mGMap<int, mGSet<mGState> >::mGMapItator it = labelSets.begin();
				it != labelSets.end(); it++)
		{

			mGInt temp_int((*it).second);
			int index_int = bigInts.member(temp_int);
			if (index_int < 0)
			{

				setsToProcess += (*it).second;
				bigInts += temp_int;
				mGTransition t(index, (*it).first, bigInts.size() - 1);
				dfa.addTransition(t);

			}
			else
			{
				mGTransition t(index, (*it).first, index_int);
				dfa.addTransition(t);
			}
		}

		index++;
	}
//now do the start and end state
	mGState s(0);
	dfa.addStart(0);
	mGSet<mGState> originalFinalSet = nfa.getFinalStates();

	for (int a = 0; a < setsToProcess.size(); a++)
	{
		mGSet<mGState> inter;
		mGArray<mGState>::getIntersection(originalFinalSet, setsToProcess[a],
				inter);

		if (inter.size() > 0)
			dfa.addFinal(a);

	}

}

}// namespace minigrail
