/*
    file:mgNFA.cpp
    Project: miniGrail
    License: LGPLv2.1 http://www.gnu.org/licenses/lgpl-2.1.html

    Created by: by Rui Zhou
    Maintainer: Cezar Campeanu <ccampeanu@upei.ca>


	Modified by Brandon Stewart July 2015
	  - Added namespace minigrail

*/


#include <assert.h>


namespace minigrail
{

//add a transition
//update start and final at the same time
void mGNFA::addTransition(const mGTransition& t)
{
	transitions += t;
	/*
	 if(t.getStart().isStart())
	 {
	 startStates+=t.getStart();
	 }

	 if(t.getStart().isFinal())
	 {
	 finalStates+=t.getStart();
	 }

	 if(t.getEnd().isStart())
	 {
	 startStates+=t.getEnd();
	 }

	 if(t.getEnd().isFinal())
	 {
	 finalStates+=t.getEnd();
	 }
	 */
}

//add a startState
//add a startState
void mGNFA::addStart(const mGState& s)
{
//assert(s.isStart());

	startStates += s;

}

//add a finalState
void mGNFA::addFinal(const mGState& s)
{
//assert(s.isFinal());

	finalStates += s;
}

//clear this NFA
void mGNFA::clear()
{
	startStates.clear();
	finalStates.clear();
	transitions.clear();
}

//gather the states
void mGNFA::gatherStates(mGSet<mGState>& set) const
{
	for (int a = 0; a < transitions.size(); a++)
	{
		set.disjoint_union(transitions[a].getStart());
		set.disjoint_union(transitions[a].getEnd());

	}
	set.unique_array();
}

//gather the Labels
void mGNFA::gatherLabels(mGSet<int> & set) const
{
	for (int a = 0; a < transitions.size(); a++)
	{
		set.disjoint_union(transitions[a].getLabel());
	}
	set.unique_array();
}

//attach the transitions
void mGNFA::attach()
{
	bool debug = false;
	if (DEBUG_TRIGGERED(DEBUG_mGNFA_attach) || DEBUG_TRIGGERED(DEBUG_ALL))
	{
		debug = true;
		std::cout << "Debug output for mGNFA::attach() is enabled."
				<< std::endl;

	}

	if (isAttached)
		return;
	for (int a = 0; a < transitions.size(); a++)
	{
		//transitionsByState[transitions[a].getStart()] += transitions[a];
		transitionsByState[transitions[a].getStart()].disjoint_union(
				transitions[a]);
//should not have duplicated lines
	}
	isAttached = true;
	if (debug)
	{
		std::cout << "The attached dictionary" << std::endl
				<< transitionsByState << std::endl;
	}

}

// attach the transitions  inverse
// do not write into transitionsbysatates, write into
//the passed in parameter

void mGNFA::attach_inv(mGMap<mGState, mGSet<mGTransition> >& inv)
{
	bool debug = false;
	if (DEBUG_TRIGGERED(DEBUG_mGNFA_attach) || DEBUG_TRIGGERED(DEBUG_ALL))
	{
		debug = true;
		std::cout << "Debug output for mGNFA::attach_inv() is enabled."
				<< std::endl;

	}

	for (int a = 0; a < transitions.size(); a++)
	{
		mGTransition rev;
		transitions[a].getReverseCopy(rev);

		inv[rev.getStart()].disjoint_union(rev);

	}

	if (debug)
	{
		std::cout << "The invserse attached dictionary" << std::endl << inv
				<< std::endl;
	}

}

/* idea:
 *
 cezar@sun11.math.upei.ca
 4:57 PM (7 hours ago)

 to me
 ok.
 My opinion is that you don't need to sort all input data, just for each
 state (create each state - symbols and everything while they appear in the
 input data).

 theoretically it should be faster than transitions.unqiue_array();
 however not in practice
 */

void mGNFA::unqiueTransitionsByAttach()
{
	attach();
//remove duplicates in each attached set to every state
	//also write back
	transitions.clear();

	mGMap<mGState, mGSet<mGTransition> >::mGMapItator it;
	for (it = transitionsByState.begin(); it != transitionsByState.end(); it++)
	{

		((*it).second).unique_array();

		for (int a = 0; a < ((*it).second).size(); a++)
		{
			transitions.disjoint_union(((*it).second)[a]);

		}

	}

}

void mGNFA::writeBackFromAttached()
{
	transitions.clear();

	mGMap<mGState, mGSet<mGTransition> >::mGMapItator it;
	for (it = transitionsByState.begin(); it != transitionsByState.end(); it++)
	{

		for (int a = 0; a < ((*it).second).size(); a++)
		{
			transitions.disjoint_union(((*it).second)[a]);

		}

	}

}

//clear the attach
void mGNFA::clearAttach()
{
	if (!isAttached)
		return;

	isAttached = false;
	transitionsByState.clear();
}

mGSet<mGTransition>& mGNFA::getTransitionsStartFrom(const mGState& s) //no const?
{
	return transitionsByState[s];
}

/*
 const mGSet<mGTransition>& mGNFA::getTransitionsStartFrom( const mGState& s)  const //no const?
 {
 return  transitionsByState[s];
 }
 */
void mGNFA::rename()
{
	if (statesRenamed)
		return;

//get the original states
	mGSet<mGState> states;
	this->gatherStates(states);
	states.heapSort();
	statesMapping.clear();
//mGSet up the mapping
	for (int a = 0; a < states.size(); a++)
	{
		statesMapping += states[a].getNumber();
	}

//rename the machines
	for (int a = 0; a < startStates.size(); a++)
	{
		startStates[a].setNumber(
				statesMapping.member(startStates[a].getNumber()));
	}

	for (int a = 0; a < finalStates.size(); a++)
	{
		finalStates[a].setNumber(
				statesMapping.member(finalStates[a].getNumber()));
	}

	for (int a = 0; a < transitions.size(); a++)
	{
		transitions[a].getStart().setNumber(
				statesMapping.member(transitions[a].getStart().getNumber()));
		transitions[a].getEnd().setNumber(
				statesMapping.member(transitions[a].getEnd().getNumber()));

	}

	statesRenamed = true;
}

void mGNFA::cancelRename() //cancelRename if still meaningful
{
	if (!statesRenamed)
		return;

	std::cout << "NOT IMPLEMENTED FUNCTION BEING CALLED";

}

mGNFA& mGNFA::select(const mGState& r, int opt, mGNFA& nfa) const
{

	int i;

	nfa.clear();

	// if sorted and we only want source element, do binary search
	if (transitions.isSorted() && opt == SOURCE)
	{
		if ((i = find_first(r)) == -1)
			return nfa;

		for (; i < size(); ++i)
		{
			if (!(transitions[i].getStart() == r))
				break;
			nfa.transitions.disjoint_union(transitions[i]);
		}

		return nfa;
	}

	// if not sorted, do it by linear pass
	if (opt == EITHER)
	{
		for (i = 0; i < size(); ++i)
			if (transitions[i].getStart() == r || transitions[i].getEnd() == r)
				nfa.transitions.disjoint_union(transitions[i]);
	}

	if (opt == SINK)
	{
		for (i = 0; i < size(); ++i)
			if (transitions[i].getEnd() == r)
				nfa.transitions.disjoint_union(transitions[i]);
	}

	if (opt == SOURCE)
	{
		for (i = 0; i < size(); ++i)
			if (transitions[i].getStart() == r)
				nfa.transitions.disjoint_union(transitions[i]);
	}

	return nfa;
}

/***************************************************************************

 //test Imports fromn grail+

 int mGNFA::find_first(const mGState& r) const

 Description:
 This function is a protected member of the template class mGNFA (declared in
 classes/re/mGNFA.h).  If the mGSet of arcs is sorted, this function can be
 used to find the first occurrence of the given source mGState in the mGSet
 of arcs for this mGNFA.  A recursive binary search is performed to do the
 search.  Note that it is the programmer's responsibility to make sure
 that the arcs are sorted before this function is called since no
 checking is done.  The function returns the index of the first arc
 containing the mGState r as its source, or -1 if no such arc is found.

 This function can be called in the following way:
 result = this_fm.find_first(state1);

 Parameters:
 const mGState& r - the source mGState to find

 Return Value:
 int - the index of the first arc containing the mGState r as its source,
 or -1 if no such arc is found

 ***************************************************************************/

int mGNFA::find_first(const mGState& r) const
{

	int left = 0;
	int right = transitions.size();
	int probe;
	mGState s1;

	for (;;)
	{
		probe = (right + left) / 2;
		s1 = transitions[probe].getStart();

		// back up to first occurrence
		if (s1 == r)
			for (;;)
			{
				if (probe == 0)
					return probe;

				s1 = transitions[--probe].getStart();

				if (s1 != r)
					return ++probe;
			}

		if (probe == left)
		{
			if (probe + 1 < transitions.size()
					&& transitions[probe + 1].getStart() == r)
				return probe + 1;
			break;
		}

		if (s1 < r)
			left = probe;
		else
			right = probe;
	}

	return -1;
}

/***************************************************************************

 void mGNFA::reachable_states(mGSet<mGState>& s) const

 Description:
 This function is a public member of the template class mGNFA (declared in
 classes/re/mGNFA.h).  This function extracts the mGSet states reachable
 from a start mGState in this mGNFA and returns them as the parameter mGSet, s.
 This mGNFA is unchanged.

 This function can be called in the following way:
 this_fm.reachable_states(state_set);

 Parameters:
 mGSet<mGState>& s - after execution contains the mGSet of reachable states

 Return Value:  none

 ***************************************************************************/

void mGNFA::reachable_states(mGSet<mGState>& s)
{

	int i;

	mGSet<mGState> wave = startStates;
	s = wave;

	s.heapSort();
	transitions.heapSort();

	for (;;)
	{
		// find all targets reachable in one instruction

		mGSet<mGState> targets;
		mGSet<mGState> temp2;
		mGArray<mGState> temp3;
		mGNFA temp;

		temp3.clear();

		for (i = 0; i < wave.size(); ++i)
		{
			select(wave[i], SOURCE, temp);
			temp.sinks(temp2);
			temp2.heapSort();
			temp3 += temp2;
		}

		targets = (temp3); //TODO from list in grail?

		// if we've found new states, they will be used on
		// the next iteration

		wave.clear();
		for (i = 0; i < targets.size(); ++i)
			if (s.member(targets[i]) < 0)
				wave.disjoint_union(targets[i]);

		if (wave.size() == 0)
			break;

		wave.heapSort();
		s.mergeSortedArrays(wave);
	}

}

//get the sinks
mGSet<mGState>& mGNFA::sinks(mGSet<mGState>& s) const
{
	s.clear();
	for (int i = 0; i < size(); ++i)
		s.disjoint_union(transitions[i].getEnd());

	s.unique_array();
	return s;
}

void mGNFA::reachable()
{

	int i;
	mGState m1;
	mGState m2;
	mGNFA result;
	mGSet<mGState> r_states;

	reachable_states(r_states);
	//std::cout << r_states << std::endl;

	result.startStates = startStates;

	//std::cout << result.startStates << std::endl;

	// keep only reachable final states

	result.finalStates.intersect(finalStates, r_states);

	//std::cout << result.finalStates << std::endl;

	//std::cout << transitions << std::endl;

	//DEBUG_TRIGGER(DEBUG_mGArray_member);
	for (i = 0; i < size(); ++i)
	{
		m1 = transitions[i].getStart();
		m2 = transitions[i].getEnd();
		if ((r_states.member(m1) >= 0) && (r_states.member(m2) >= 0))
		{
			//std::cout << "found" << std::endl;
			result.transitions.disjoint_union(transitions[i]);
		}

	}

	//std::cout << result.transitions << std::endl;

	*this = result;

}

//reverse this finite machine
mGNFA& mGNFA::reverse()
{

	//reverse the start and finals
	startStates.swap(finalStates);

	for (int a = 0; a < transitions.size(); a++)
	{
		transitions[a].reverse();

	}
	transitions.setSorted(false);

	return *this;
}

//isc ompleted
bool mGNFA::isCompleted()
{
	attach();

	mGMap<mGState, mGSet<mGTransition> >::mGMapItator it;

	for (it = transitionsByState.begin(); it != transitionsByState.end(); it++)
	{
		mGArray<int> labels;
		for (int a = 0; a < (*it).second.size(); a++)
		{
			labels += (*it).second[a].getLabel();
		}

		labels.unique_array();
		if (labels.size() < mGIntTable.size())
			return false;

	}
	return true;
}


} //namespace minigrail
