/*
*    Language Investigator and Stractural Analyzer (LISA) -
*    A descriptive programming language
* LICENCE :             LGPL
*
* FILE:		icg.h
*
* PURPOSE:	Intermidiate Code Generator (ICG) definitions and declarations.
*
* CREATED AT:	November 15, 2011
*
* AUTHOR: 	Seyed Pooria Madani Kochak (s.p.madani@gmail.com)
*
* Project Manager:	Dr. Cezar Campeanu (ccampeanu@upei.ca)
*/

/*
* This file holds declarations for Intermediate Code Generator.
* All the intermediate code for each grammar production is controlled by
* this file.
*
* Revision History
* --------------------------------------------------------------------------
* Seyed Pooria Madani - Creation							November 15, 2011
* Brandon Stewart     - Namespace support, and restructuring, details in
*                     "lisa-changelist.txt".                      June/July 2015
*/

#ifndef ICG_H
#define ICG_H
#include <stdio.h>
#include <regex.h>
#include <string.h>
#include <stdlib.h>
#include "abstractsyntaxtree.h"

FILE *templatefile,*generatedfile;
unsigned char *code; //Keep the code generated.
long codeSize; //Keep track of the code size generated

/*
 * Will open template file, initilize "codeSize" and "code"
 */
void init_icg(void);

/*
 * Will generate intermediate code that is related to
 * declaration part (ie. declare{    } )
 */
void declaration_icg(int);

/*
 * Same as above, but is called when namespace::type is parsed
 */
void namespace_declaration_icg(int,int,int);


/*
 * Will finilize the generated code and write it to "generatedfile"
 */
void close_icg(void);


/*
 * This will generate code for "constant" production intthe grammar
 */
struct Constant *constant_icg(int);
struct Constant *constant_next_icg(int);
/*
 * This will generate code for "constant" production intthe grammar
 */
struct Constant *constant_wrap_function(struct Function*);

struct Constant *constant_wrap_variable(struct Variable*);
/*
 * This will generate code for "expression" production intthe grammar
 */
struct Expression *expression_icg(struct Variable*,int,struct Expression*);

/*
 * This will generate code for "expression" production intthe grammar
 */
struct Expression *expression_wrap_simpleexpression_icg(struct Simple_Expression*);

/*
 * This will generate code for "factor" production intthe grammar
 */
struct Factor *factor_icg(struct Simple_Expression*);

/*
 * This will generate code for "factor" production intthe grammar
 */
struct Factor *factor_wrap_constant(struct Constant*);

/*
 * This will generate code for "term" production intthe grammar
 */
struct Term *term_icg(struct Term*,int,struct Factor*);

/*
 * This will generate code for "term" production intthe grammar
 */
struct Term	*term_wrap_factor(struct Factor*);

/*
 * This will generate code for "addexpression" production intthe grammar
 */
struct Add_Expression *addexpression_icg(struct Add_Expression*,int,struct Term*);

/*
 * This will generate code for "addexpression" production intthe grammar
 */
struct Add_Expression *addexpression_wrap_term(struct Term*);

/*
 * This will generate code for "relationexpression" production intthe grammar
 */
struct Relation_Expression *relationexpression_icg(struct Add_Expression*,int,struct Add_Expression*);

/*
 * This will generate code for "relationexpression" production intthe grammar
 */
struct Relation_Expression *relationexpression_wrap_addexpression(struct Add_Expression*);

/*
 * This will generate code for "unaryrelationexpression" production intthe grammar
 */
struct Unary_Relation_Expression *unaryrelationexpression(struct Unary_Relation_Expression *);

/*
 * This will generate code for "unaryrelationexpression" production intthe grammar
 */
struct Unary_Relation_Expression *unaryrelationexpression_wrap_relationexpression(struct Relation_Expression*);

/*
 * This will generate code for "orexpression_" production intthe grammar
 */
struct Or_Expression *orexpression_icg(struct Or_Expression*,int, struct Unary_Relation_Expression*);

/*
 * This will generate code for "orexpression_" production intthe grammar
 */
struct Or_Expression *orexpression_wrap_unaryrelationexpression(struct Unary_Relation_Expression*);

/*
 * This will generate code for "simpleexpression" production intthe grammar
 */
struct Simple_Expression *simpleexpression_icg(struct Simple_Expression*,int,struct Or_Expression*);

/*
 * This will generate code for "simpleexpression" production intthe grammar
 */
struct Simple_Expression *simpleexpression_wrap_orexpression(struct Or_Expression*);

/*
 * This will generate code for "function" production intthe grammar
 */
struct Function *function_icg(int,struct Parameter_List*);

/*
 * This will generate code for "function" production intthe grammar, with a namespace
 */
struct Function *namespace_function_icg(int, int, int,struct Parameter_List*);

/*
 * This will generate code for "parameterlist" production intthe grammar
 */
struct Parameter_List *parameterlist_icg(struct Parameter_List*, struct Simple_Expression*);

/*
 * This will generate code for "parameterlist" production intthe grammar
 */
struct Parameter_List *parameterlist_simpleexpression_icg(struct Simple_Expression*);

/*
 * This will generate code for "parameterlist" production intthe grammar
 */
struct Parameter_List *parameterlist_type_icg(int);

/*
 * This will generate code for "variable" production intthe grammar
 */
struct Variable *variable_icg(int, int);

/*
 * This will generate code for "ifstatement" production intthe grammar
 */
unsigned char *ifstatement_icg(struct Expression*,unsigned char*);

/*
 * This will generate code for "compundstatement" production intthe grammar
 */
unsigned char *compundstatement_icg(unsigned char*);

/*
 * This will generate code for "statements" production intthe grammar
 */
unsigned char *statements_icg(unsigned char *, unsigned char *);

/*
 * This will generate code for "whilestatement" production intthe grammar
 */
unsigned char *whilestatement_icg(struct Expression*, unsigned char *);

unsigned char *generatingstatement_icg(int,int,int,unsigned char*);

unsigned char *namespace_generatingstatement_icg(int,int,int,int,unsigned char*);

/*
 * This will generate code for "breakstatement" production intthe grammar
 */
unsigned char *breakstatement_icg(int);

/*
 * This will generate code for "continuestatement" production intthe grammar
 */
unsigned char *continuestatement_icg(int);

/*
 * This will generate code for "expressionstatement" production intthe grammar
 */
unsigned char *expressionstatement_icg(unsigned char*);

/*
 * This will print all the generated statements into the source template.
 */
void print_statement(unsigned char *);

unsigned char *getHeapObjectName();


/*
 * This will generate code for "namespace" production in the grammar
 */
void using_declaration_icg(int);


#endif
