%{
/*FILE: lex.l
* LICENCE :             LGPL
* AUTHOR: Pooria Seyed Madani
* PROJECT: LISA
* MAINTEINER: Cezar Campeanu <ccampeanu@upei.ca>
*
* Modified by:
*
*	Brandon Stewart - added "using" and "::"  June/July 2015
*/
#include <stdlib.h>
#include "y.tab.h"


void yyerror(char *);

int count(int);

int lineNumber = 0;
int charNumber = 0;


%}


WHITESPACE [ ]
TABSPACE   [\t]+
IDENTIFIER [a-zA-Z][a-z0-9A-Z_]*
NEWLINE [\r\n]
COMMENT \/\/.*
SINGLECHAR [-,$()<>=\+\;{}\.\!\*\/\%]
STRING	  L?\"(\\.|[^\\"])*\"
%%
{COMMENT}	;/*skip the comments*/
{NEWLINE} 		{charNumber=0; lineNumber++; }
{WHITESPACE}	{count(strlen(yytext));} /* skip whitespaces and tabs */
{TABSPACE}		{count(strlen(yytext)* 8);} /* skip whitespaces and tabs */
"int"           {count(strlen(yytext));yylval.sIndex=add_character_symbol_node(yytext); return INT;}
"bool"			{count(strlen(yytext));yylval.sIndex=add_character_symbol_node(yytext); return BOOL;}
"dfa"           {count(strlen(yytext));yylval.sIndex=add_character_symbol_node(yytext); return DFA;}
"nfa"           {count(strlen(yytext));yylval.sIndex=add_character_symbol_node(yytext); return NFA;}
"regex"         {count(strlen(yytext));yylval.sIndex=add_character_symbol_node(yytext); return REGEX;}
"declare"       {count(strlen(yytext));yylval.sIndex=add_character_symbol_node(yytext); return DECLARE;}
"program"       {count(strlen(yytext));yylval.sIndex=add_character_symbol_node(yytext); return PROGRAM;}
"using"         {count(strlen(yytext));yylval.sIndex=add_character_symbol_node(yytext); return USING;}
"while"         {count(strlen(yytext));yylval.sIndex=add_character_symbol_node(yytext); return WHILE;}
"if"            {count(strlen(yytext));yylval.sIndex=add_character_symbol_node(yytext); return IF;}
"else"          {count(strlen(yytext));yylval.sIndex=add_character_symbol_node(yytext); return ELSE;}
"true"			{count(strlen(yytext));yylval.sIndex=add_character_symbol_node(yytext); return TRUE;}
"false"			{count(strlen(yytext));yylval.sIndex=add_character_symbol_node(yytext); return FALSE;}
"break"			{count(strlen(yytext));yylval.sIndex=add_character_symbol_node(yytext); return BREAK;}
"continue"		{count(strlen(yytext));yylval.sIndex=add_character_symbol_node(yytext); return CONTINUE;}
"next"			{count(strlen(yytext));yylval.sIndex=add_character_symbol_node(yytext); return NEXT;}
"hasnext"		{count(strlen(yytext));yylval.sIndex=add_character_symbol_node(yytext); return HASNEXT;}
"length"		{count(strlen(yytext));yylval.sIndex=add_character_symbol_node(yytext); return LENGTH;}
"generate"		{count(strlen(yytext));yylval.sIndex=add_character_symbol_node(yytext); return GENERATE;}
"enumerate"		{count(strlen(yytext));yylval.sIndex=add_character_symbol_node(yytext); return ENUMERATE;}
"string"		{count(strlen(yytext));yylval.sIndex=add_character_symbol_node(yytext); return STRING;}
"random"		{count(strlen(yytext));yylval.sIndex=add_character_symbol_node(yytext); return RANDOM;}
"alphabet"		{count(strlen(yytext));yylval.sIndex=add_character_symbol_node(yytext); return ALPHABET;}
"type"			{count(strlen(yytext));yylval.sIndex=add_character_symbol_node(yytext); return TYPE;}
"=="			{count(strlen(yytext));yylval.sIndex=add_character_symbol_node(yytext); return EQ;}
"+="			{count(strlen(yytext));yylval.sIndex=add_character_symbol_node(yytext); return AE;}
"-="			{count(strlen(yytext));yylval.sIndex=add_character_symbol_node(yytext); return ME;}
"*="			{count(strlen(yytext));yylval.sIndex=add_character_symbol_node(yytext); return TE;}
"/="			{count(strlen(yytext));yylval.sIndex=add_character_symbol_node(yytext); return DE;}
"||"			{count(strlen(yytext));yylval.sIndex=add_character_symbol_node(yytext); return OR;}
"&&"			{count(strlen(yytext));yylval.sIndex=add_character_symbol_node(yytext); return AND;}
"<="			{count(strlen(yytext));yylval.sIndex=add_character_symbol_node(yytext); return LE;}
">="			{count(strlen(yytext));yylval.sIndex=add_character_symbol_node(yytext); return GE;}
"!="			{count(strlen(yytext));yylval.sIndex=add_character_symbol_node(yytext); return NE;}
"::"			{count(strlen(yytext));yylval.sIndex=add_character_symbol_node(yytext); return NAMESPACE;}

"["				{count(1);yylval.sIndex=add_character_symbol_node(yytext); return *yytext; }
"]"				{count(1);yylval.sIndex=add_character_symbol_node(yytext); return *yytext; }


{IDENTIFIER} 	{
				  char *str = (char *) malloc(sizeof(char) * (yyleng+1));
				  memcpy(str,yytext,yyleng+1);
				  count(strlen(yytext)); yylval.sIndex =   add_id_symbol_node(str);
				  return ID;
				}

{STRING}		{
					char *str = (char *) malloc(sizeof(char) * (yyleng+1));
					memcpy(str,yytext,(yyleng+1));
					count(strlen(yytext)); yylval.sIndex = add_stringliterals_symbol_node(str);
					return ID;
				}

[\+\-]?[0-9][0-9]*     {char *str = (char *) malloc(sizeof(char) * (yyleng+1));
					memcpy(str,yytext,(yyleng+1));

					count(strlen(yytext)); yylval.iValue = add_number_symbol_node(str,atoi(yytext));

				 return INTEGER;
				}


{SINGLECHAR}	{count(1);yylval.sIndex=add_character_symbol_node(yytext); return *yytext; }
.		;
%%
int yywrap(void) {
	return(1);
}

int count(int i){
	charNumber += i;
}
