%{
/*
 *   FILE: parser.y
 *
 *
 *
 *  AUTHOR: Pooria Madani
 *  PROJECT: LISA
 *  MANTAINER: Cezar Campeanu
 *
 *
 *  Modified By:
 *
 *	Brandon Stewart - modifed grammar to support namespaces   June/July 2015
 *  Brandon Stewart - changed generators to accept variable parameters, and constants.
 * 						(generator(random, x, 2) used to produce a syntax error)
 *															  October 2015
*/
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include "lex.yy.c"
#include "../abstractsyntaxtree.h"


int yylex(void);
#define DEBUG_MODE 	7000
#define NAME_MODE 	7001

unsigned char *output_name;
unsigned char *input_name;
unsigned char *lisa_code;
FILE *source_file;
extern FILE * yyin;

void yyerror(char *s);
int debugMode = 0;
int fileName = 0;
extern int lineNumber;
extern int charNumber;
%}

%union{
	int 								iValue; 	/* integer value */
	int 								sIndex; 	/* symbol table index */
	struct nodeType 					*nPtr;	/* node pointer */
	unsigned char						*strPtr;
	struct Expression					*expr;
	struct Simple_Expression  			*simple_expr;
	struct Or_Expression 	  			*or_expr;
	struct Unary_Relation_Expression 	*ur_expr;
	struct Relation_Expression 			*re_expr;
	struct Add_Expression				*add_expr;
	struct Term 						*term_node;
	struct Factor 						*factor_node;
	struct Constant 					*constant_node;
	struct Function 					*function_node;
	struct Parameter_List 				*param_list;
	struct Variable						*var;

};

/************************************************
				Declaring types
*************************************************/

%type  <sIndex> 						type;
%type  <sIndex>							generatortype;
%type  <constant_node> 					constant;
%type  <factor_node> 					factor;
%type  <term_node> 						term;
%type  <add_expr> 						addexpression;
%type  <re_expr> 						relationexpression;
%type  <ur_expr> 						unaryrelationexpression;
%type  <or_expr> 						orexpression;
%type  <simple_expr> 					simpleexpression;
%type  <expr> 							expression;
%type  <strPtr> 						compundstatement;
%type  <strPtr> 						statements;
%type  <strPtr> 						whilestatement;
%type  <strPtr>							generatingstatement;
%type  <sIndex>							generatorcall;
%type  <strPtr> 						ifstatement;
%type  <function_node> 					function;
%type  <param_list> 					parameterlist;
%type  <sIndex> 						relop;
%type  <sIndex> 						exprop;
%type  <sIndex> 						addop;
%type  <var> 							variable;
%type  <strPtr> 						expressionstatement;
%type  <strPtr> 						statement;
%type  <sIndex> 						multop;
%type  <iValue> 						number;


/************************************************
				Declaring Tokens
*************************************************/
%token <iValue> 				INTEGER
%token <sIndex> 				ID
%token <sIndex> 				AND
%token <sIndex> 				OR
%token <sIndex> 				STRINGLITERAL
%token <sIndex>					CONTINUE
%token <sIndex> 				BREAK
%token <iValue>INT DFA NFA REGEX DECLARE PROGRAM USING WHILE IF ELSE AE TE ME DE BOOL TRUE FALSE NEXT HASNEXT GENERATE LENGTH TYPE ALPHABET ENUMERATE RANDOM STRING NAMESPACE
%left AND OR
%left EQ GE LE NE ';' ',' '$'
%left '+' '-'
%left '*' '/' '%'
%left '!'
%left '(' ')'
%left '[' ']'
%%



program:
	declarationblock programblock				{if(debugMode)if(debugMode)printf("-1\n"); }
	;

declarationblock:
	DECLARE '{' declarations  '}'  			{if(debugMode)if(debugMode)printf("%d-2\n",lineNumber);}
	;


declarations:
	declarations declaration				{if(debugMode)if(debugMode)printf("%d-3a\n",lineNumber);}
	| /* NULL */							{if(debugMode)if(debugMode)printf("%d-3b\n",lineNumber);}
	;

declaration:
	type ID ';'								{set_variable_type($2,$1);
												declaration_icg($2);if(debugMode)printf("%d***%d***",$2,$1);
												if(debugMode)printf("%d-4a\n",lineNumber);}

	|type '[' INTEGER ']' ID ';'			{set_array_type($5,$1,$3);declaration_icg($5);if(debugMode)printf("%d-4b\n",lineNumber);}

	|ID NAMESPACE type '[' INTEGER ']' ID ';' {namespace_set_array_type($1, $7,$3,$5);
												namespace_declaration_icg($1, $2,$7);
												if(debugMode)printf("%d-4c\n",lineNumber);}

	//NAMESPACES
	|ID NAMESPACE type ID ';'				{ //set_variable_type($4,$3);
										namespace_set_variable_type($1,$4,$3);
										//declaration_icg($4);
										namespace_declaration_icg($1, $2, $4);
										if(debugMode)printf("Namespace: %d***%d***\n",$1,$2);
										if(debugMode)printf("%d***%d***",$4,$3);
										if(debugMode)printf("%d-4d\n",lineNumber); }

	| USING ID ';' 						{  if(debugMode)printf("%d-4e\n",lineNumber);
										using_declaration_icg($2);
										}
	;

type:
	  INT		       			{$$ = yylval.iValue;if(debugMode)printf("%d-5a\n",lineNumber);}
	| DFA						{$$ = yylval.iValue;if(debugMode)printf("%d-5b\n",lineNumber);}
	| NFA						{$$ = yylval.iValue;if(debugMode)printf("%d-5c\n",lineNumber);}
	| REGEX						{$$ = yylval.iValue;if(debugMode)printf("%d-5d\n",lineNumber);}
	| BOOL						{$$ = yylval.iValue;if(debugMode)printf("%d-5e\n",lineNumber);}
	| STRING					{$$ = yylval.iValue;if(debugMode)printf("%d-5f\n",lineNumber);}
	;



programblock:
	PROGRAM '{' statements '}'		{print_statement($3);if(debugMode)printf("%d-6\n**p*\n%s\n*s**",lineNumber, $3);}
	;

statements:
	statements statement			{$$ = statements_icg($1,$2);if(debugMode)printf("%d-7a\n",lineNumber);}
	| /* NULL */ /*statement*/		{$$ = " ";if(debugMode)printf("%d-7b\n",lineNumber);}
	;
statement:
	expressionstatement				{$$ = $1;if(debugMode)printf("%d-8a\n",lineNumber);}
	| ifstatement					{$$ = $1;if(debugMode)printf("%d-8b\n",lineNumber);}
	| compundstatement				{$$ = $1;if(debugMode)printf("%d-8c\n",lineNumber);}
	| whilestatement				{$$ = $1;if(debugMode)printf("%d-8d\n",lineNumber);}
	| generatingstatement			{$$ = $1;if(debugMode)printf("%d-8e\n",lineNumber);}
	| BREAK	';'						{$$ = breakstatement_icg($1);if(debugMode)printf("%d-8f\n",lineNumber);}
	| CONTINUE ';'					{$$ = continuestatement_icg($1);if(debugMode)printf("%d-8g\n",lineNumber);}

	/*|
	| selectionstatement
	| iterationstatement
	| returnstatement
	| breakstatement*/
	;

generatingstatement:
//	GENERATE '(' generatortype ',' INTEGER ',' INTEGER ')' statement  {$$ = generatingstatement_icg($3,$5,$7,$9);if(debugMode)printf("%d-26\n",lineNumber);}
//	| ID NAMESPACE GENERATE '(' generatortype ',' INTEGER ',' INTEGER ')' statement  {$$ = namespace_generatingstatement_icg($1,$5,$7,$9,$11);
//																		if(debugMode)printf("%d-26b\n",lineNumber);}
	generatorcall '(' generatortype ',' number ',' number ')' statement  {$$ = generatingstatement_icg($3,$5,$7,$9);if(debugMode)printf("%d-26c\n",lineNumber);}
	 | ID NAMESPACE generatorcall '(' generatortype ',' number ',' number ')' statement  {$$ = namespace_generatingstatement_icg($1,$5,$7,$9,$11);
																		if(debugMode)printf("%d-26d\n",lineNumber);}
	;
generatortype:
	RANDOM						{$$ = yylval.sIndex;}
	| ENUMERATE					{$$ = yylval.sIndex;}
	;


generatorcall:
	GENERATE {$$ = yylval.sIndex; addGeneratorToStack(); if(debugMode)printf("SET GEN ID: %d\n", yylval.sIndex);}
	;

number:
	INTEGER {$$ = yylval.sIndex; if(debugMode)printf("%d-48a\n",lineNumber);}
	| ID	{$$ = yylval.sIndex; if(debugMode)printf("%d-48b\n",lineNumber);}
	;
expressionstatement:
	expression ';'					{$$ = expressionstatement_icg($1->code);if(debugMode)printf("%d-9a\n",lineNumber);}
	/*| ';'						{if(debugMode)printf("%d-9b\n",lineNumber);}*/
	;


ifstatement:
	IF '(' expression ')' statement		{ $$ = ifstatement_icg($3,$5);if(debugMode)printf("%d-10a\n",lineNumber);}
	/*| IF '(' expression ')' statement ELSE statement {if(debugMode)printf("%d-10b\n",lineNumber);}*/
	/* ASk Dr. Campeanu about conflic on else */
	;

compundstatement:
	'{' statements '}'						{$$ = compundstatement_icg($2);if(debugMode)printf("%d-11\n",lineNumber);}
	;

whilestatement:
	WHILE '(' expression ')' statement		{$$ = whilestatement_icg($3,$5);if(debugMode)printf("%d-25\n",lineNumber);}
	;

expression:
	variable exprop expression		{$$ = expression_icg($1,$2,$3);if(debugMode)printf("*******Generated Expression %d %d %s\n",$1,$2,$3->code);if(debugMode)printf("%d-12a\n",lineNumber);}
	| simpleexpression				{$$ = expression_wrap_simpleexpression_icg($1);if(debugMode)printf("%d-12g\n",lineNumber);}
	;
exprop:
	  '='							{$$ = yylval.sIndex;}
	| ME /* -= */					{$$ = yylval.sIndex;}
	| TE /* *= */					{$$ = yylval.sIndex;}
	| DE /* /= */					{$$ = yylval.sIndex;}
	| AE /* += */ 					{$$ = yylval.sIndex;}
	;
simpleexpression:
	simpleexpression OR orexpression {$$ = simpleexpression_icg($1,$2,$3);if(debugMode)printf("%d-13a\n",lineNumber);}
	| orexpression			 		 {$$ = simpleexpression_wrap_orexpression($1);if(debugMode)printf("%d-13b\n",lineNumber);}
	;


orexpression:
	orexpression AND unaryrelationexpression 	{$$ = orexpression_icg($1,$2,$3);if(debugMode)printf("%d-14a\n",lineNumber);}
	| unaryrelationexpression 					{$$ = orexpression_wrap_unaryrelationexpression($1);if(debugMode)printf("%d-14b\n",lineNumber);}
	;

unaryrelationexpression:
	'!' unaryrelationexpression			{$$ = unaryrelationexpression($2);if(debugMode)printf("%d-15a\n",lineNumber);}
	| relationexpression 			  	{$$ = unaryrelationexpression_wrap_relationexpression($1);if(debugMode)printf("%d-15b\n",lineNumber);}
	;

relationexpression:
	addexpression relop addexpression		{$$ = relationexpression_icg($1,$2,$3);if(debugMode)printf("%d-16a\n",lineNumber);}
	| addexpression							{$$ = relationexpression_wrap_addexpression($1); if(debugMode)printf("%d-16b\n",lineNumber);}
	;
relop:
	 LE /* <= */					{$$=yylval.sIndex;if(debugMode)printf("%d-17a\n",lineNumber);}
	| GE /* >= */					{$$=yylval.sIndex;if(debugMode)printf("%d-17b\n",lineNumber);}
	| EQ /* == */					{$$=yylval.sIndex;if(debugMode)printf("%d-17c\n",lineNumber);}
	| NE /* != */					{$$=yylval.sIndex;if(debugMode)printf("%d-17d\n",lineNumber);}
	| '>'							{$$=yylval.sIndex;if(debugMode)printf("%d-17e\n",lineNumber);}
	| '<'							{$$=yylval.sIndex;if(debugMode)printf("%d-17f\n",lineNumber);}
	;


addexpression:
	addexpression addop term	{$$ = addexpression_icg($1,$2,$3);if(debugMode)printf("%d-18a\n",lineNumber);}
	| term						{$$ = addexpression_wrap_term($1); if(debugMode)printf("%d-18b\n",lineNumber);}
	;

addop:
	  '-'						{$$=yylval.sIndex;if(debugMode)printf("%d-19a\n",lineNumber);}
	| '+'						{$$=yylval.sIndex;if(debugMode)printf("%d-19b\n",lineNumber);}
	;

term:
      term multop factor			{$$ = term_icg($1,$2,$3);if(debugMode)printf("%d-20a\n",lineNumber);}
	| factor					{$$ = term_wrap_factor($1); if(debugMode)printf("%d-20b\n",lineNumber);}
	;

multop:
	  '*'						{$$=yylval.sIndex;if(debugMode)printf("%d-21a\n",lineNumber);}
	| '/'						{$$=yylval.sIndex;if(debugMode)printf("%d-21b\n",lineNumber);}
	| '%'						{$$=yylval.sIndex;if(debugMode)printf("%d-21c\n",lineNumber);}
	;

factor:
	'(' simpleexpression ')' 	{$$ = factor_icg($2);if(debugMode)printf("%d-22a\n",lineNumber);}
	| constant					{ $$ = factor_wrap_constant($1); if(debugMode)printf("%d-22b\n",lineNumber);}
	;

constant:
	  INTEGER						{$$ = constant_icg(yylval.iValue);if(debugMode)printf("%d-23a\n",lineNumber);}
	| TRUE						{$$ = constant_icg(yylval.sIndex);if(debugMode)printf("%d-23b\n",lineNumber);}
	| FALSE						{$$ = constant_icg(yylval.sIndex);if(debugMode)printf("%d-23c\n",lineNumber);}
	| NEXT 						{$$ = constant_next_icg(yylval.sIndex);if(debugMode)printf("%d-23d\n",lineNumber);}
	| HASNEXT 					{$$ = constant_icg(yylval.sIndex);if(debugMode)printf("%d-23e\n",lineNumber);}
	| variable					{$$ = constant_wrap_variable($1);if(debugMode)printf("%d-23f\n",lineNumber);}
	| STRINGLITERAL				{$$ = constant_icg(yylval.sIndex);if(debugMode)printf("%d-23g\n",lineNumber);}
	| function					{$$ = constant_wrap_function($1);if(debugMode)printf("%d-23h\n",lineNumber); }
	;

function:
	  ID '(' parameterlist ')'	{ $$ = function_icg($1,$3); if(debugMode)printf("%d-30a\n",lineNumber); }
	| ID NAMESPACE ID '(' parameterlist ')'	{ $$ = namespace_function_icg($1, $2, $3, $5); if(debugMode)printf("%d-30b\n",lineNumber); }

	;
parameterlist:
	simpleexpression			{ $$ = parameterlist_simpleexpression_icg($1);if(debugMode)printf("%d-25a\n",lineNumber); }
	| type						{ $$ = parameterlist_type_icg($1);if(debugMode)printf("%d-25b\n",lineNumber);}
	|parameterlist ',' simpleexpression {$$ = parameterlist_icg($1,$3);if(debugMode)printf("%d-25c\n",lineNumber); }
	//|/* NULL */		   			{ $$ = " ";}
	;

variable:
	  ID						{$$ = variable_icg($1,-1);if(debugMode)printf("%d-24a\n",lineNumber);}
	| ID'[' INTEGER ']'         {$$ = variable_icg($1,$3);if(debugMode)printf("%d-24b\n",lineNumber);}
	| ID'[' ID ']'				{$$ = variable_icg($1,$3);if(debugMode)printf("%d-24b\n",lineNumber);}
	;




%%

extern long codeSize;


/*
unsigned char *output_name;
unsigned char *input_name;
unsigned char *lisa_code;
FILE *source_file;


    source_file = fopen("./lib/template.ccx","r");
    generatedfile = fopen("generated.cc","w");
    fseek (templatefile , 0 , SEEK_END);
    codeSize = ftell(templatefile) + 1;
    fseek (templatefile , 0 , SEEK_SET);
    code = (unsigned char *) malloc(sizeof(unsigned char) * codeSize);
    //rewind(templatefile);
    fread (code,1,codeSize,templatefile);

 */

int main(int argc, char *argv[]) {

	printf("*********************************************\n");
	printf("*             LISA Compiler                 *\n");
	printf("*                                           *\n");
	printf("*     Department of Computer Science        *\n");
	printf("*                  and                      *\n");
	printf("*         Information Technology            *\n");
	printf("*   University of Prince Edward Island      *\n");
	printf("*                                           *\n");
	printf("* AUTHOR: Seyed Pooria Madani Kochak        *\n");
	printf("*          University of P.E.I.             *\n");
	printf("*********************************************\n");



	for(int i= 1; i < argc; i++){
		char indicator = argv[i][0];
		if(indicator == '-'){
			switch(argv[i][1]){
				case 'd':
					debugMode = 1;
					break;
				case 'o':
					output_name = strtok(argv[i], ".");
					break;
				case '-':
				          if (!strcmp(argv[i],"--version"))
                                            {
                                             printf("Lisa compiler version 0.2.4\n");
                                             exit(0);
                                            }
                                        break;
				default:
					printf("%s is not known.",argv[i]);
					exit(1);
					break;
			}
		}else{
			input_name = argv[i];
		}
	}


	yyin = fopen(input_name,"r");

    output_name = strtok(input_name, ".");
	init_icg();
	init_symbol_table();



	yyparse();
	if(debugMode)
		print_symbol_table();
	close_icg();
	fclose(yyin);
	system("astyle generated.cc");
	char command[500];
	snprintf(command,500,"g++ -g -w generated.cc -lgmp -o%s.o",output_name);

	system(command);
	if(debugMode != 1){
		system("rm -f generated.cc");
	}

	delete_symbol_table();
	return 0;

}

void yyerror(char *s){
	fprintf(stdout,"Ln:%d, Col:%d : %s\n",lineNumber+1,charNumber,s);
	exit(1);
}
